@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -49,6 +49,7 @@ struct connman_ipconfig {
 	const struct connman_ipconfig_ops *ops;
 	void *ops_data;
 
+	connman_bool_t enabled;
 	enum connman_ipconfig_method method;
 	struct connman_ipaddress *address;
 	struct connman_ipaddress *system;
@@ -549,6 +550,15 @@ static void __connman_ipconfig_lower_up(struct connman_ipdevice *ipdevice)
 {
 	DBG("ipconfig ipv4 %p ipv6 %p", ipdevice->config_ipv4,
 					ipdevice->config_ipv6);
+
+	if (ipdevice->config_ipv6 != NULL &&
+			ipdevice->config_ipv6->enabled == TRUE)
+		return;
+
+	if (__connman_device_isfiltered(ipdevice->ifname) == FALSE) {
+		ipdevice->ipv6_enabled = get_ipv6_state(ipdevice->ifname);
+		set_ipv6_state(ipdevice->ifname, FALSE);
+	}
 }
 
 static void __connman_ipconfig_lower_down(struct connman_ipdevice *ipdevice)
@@ -1073,7 +1083,8 @@ unsigned int __connman_ipconfig_get_flags_from_index(int index)
 	return ipdevice->flags;
 }
 
-const char *__connman_ipconfig_get_gateway_from_index(int index)
+const char *__connman_ipconfig_get_gateway_from_index(int index,
+	enum connman_ipconfig_type type)
 {
 	struct connman_ipdevice *ipdevice;
 
@@ -1081,19 +1092,23 @@ const char *__connman_ipconfig_get_gateway_from_index(int index)
 	if (ipdevice == NULL)
 		return NULL;
 
-	if (ipdevice->ipv4_gateway != NULL)
-		return ipdevice->ipv4_gateway;
+	if (type != CONNMAN_IPCONFIG_TYPE_IPV6) {
+		if (ipdevice->ipv4_gateway != NULL)
+			return ipdevice->ipv4_gateway;
 
-	if (ipdevice->config_ipv4 != NULL &&
-			ipdevice->config_ipv4->address != NULL)
-		return ipdevice->config_ipv4->address->gateway;
+		if (ipdevice->config_ipv4 != NULL &&
+				ipdevice->config_ipv4->address != NULL)
+			return ipdevice->config_ipv4->address->gateway;
+	}
 
-	if (ipdevice->ipv6_gateway != NULL)
-		return ipdevice->ipv6_gateway;
+	if (type != CONNMAN_IPCONFIG_TYPE_IPV4) {
+		if (ipdevice->ipv6_gateway != NULL)
+			return ipdevice->ipv6_gateway;
 
-	if (ipdevice->config_ipv6 != NULL &&
-			ipdevice->config_ipv6->address != NULL)
-		return ipdevice->config_ipv6->address->gateway;
+		if (ipdevice->config_ipv6 != NULL &&
+				ipdevice->config_ipv6->address != NULL)
+			return ipdevice->config_ipv6->address->gateway;
+	}
 
 	return NULL;
 }
@@ -1240,6 +1255,7 @@ static struct connman_ipconfig *create_ipv6config(int index)
 	ipv6config->refcount = 1;
 
 	ipv6config->index = index;
+	ipv6config->enabled = FALSE;
 	ipv6config->type = CONNMAN_IPCONFIG_TYPE_IPV6;
 	ipv6config->method = CONNMAN_IPCONFIG_METHOD_AUTO;
 	ipv6config->ipv6_privacy_config = 0;
@@ -1281,6 +1297,7 @@ struct connman_ipconfig *__connman_ipconfig_create(int index,
 	ipconfig->refcount = 1;
 
 	ipconfig->index = index;
+	ipconfig->enabled = FALSE;
 	ipconfig->type = CONNMAN_IPCONFIG_TYPE_IPV4;
 
 	ipconfig->address = connman_ipaddress_alloc(AF_INET);
@@ -1672,6 +1689,8 @@ int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 	} else
 		return -EINVAL;
 
+	ipconfig->enabled = TRUE;
+
 	if (type == CONNMAN_IPCONFIG_TYPE_IPV4 &&
 					ipdevice->config_ipv4 != NULL) {
 		ipconfig_list = g_list_remove(ipconfig_list,
@@ -1740,6 +1759,8 @@ int __connman_ipconfig_disable(struct connman_ipconfig *ipconfig)
 	if (ipdevice->config_ipv4 == NULL && ipdevice->config_ipv6 == NULL)
 		return -EINVAL;
 
+	ipconfig->enabled = FALSE;
+
 	if (ipdevice->config_ipv4 == ipconfig) {
 		ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
 
@@ -2196,9 +2217,6 @@ int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 			ipconfig->ipv6_privacy_config = string2privacy(privacy);
 			g_free(pprefix);
 			g_free(privacy);
-
-			__connman_ipconfig_enable(ipconfig);
-			enable_ipv6(ipconfig);
 		}
 	}
 
